from enum import Enum

from tortoise import fields
from .base import BaseModel


class Status(BaseModel):
    name = fields.CharField(max_length=255)

    def __str__(self):
        return self.name

    class Meta(BaseModel.Meta):
        table = 'statuses'


class Type(BaseModel):
    name = fields.CharField(max_length=255)

    def __str__(self):
        return self.name

    class Meta(BaseModel.Meta):
        table = 'types'


class PriorityEnum(str, Enum):
    LOW = 'low'
    MEDIUM = 'medium'
    HIGH = 'high'


class Task(BaseModel):

    title = fields.CharField(max_length=255)
    description = fields.TextField()
    status = fields.ForeignKeyField(
        'models.Status', related_name='status_tasks',
        on_delete=fields.OnDelete.CASCADE
    )
    types = fields.ManyToManyField(
        'models.Type', related_name='type_tasks',
        through='types_tasks'
    )
    priority = fields.CharEnumField(enum_type=PriorityEnum, default=PriorityEnum.LOW)
    category = fields.ForeignKeyField(
        'models.Category', related_name='category_tasks',
        on_delete=fields.OnDelete.SET_NULL, null=True
    )

    def __str__(self):
        return self.title

    class Meta(BaseModel.Meta):
        table = 'tasks'


class UserRoleEnum(str, Enum):
    admin = "admin"
    user = "user"
    system_consumer = "system_consumer"


class User(BaseModel):
    username = fields.CharField(max_length=100, unique=True)
    name = fields.CharField(max_length=100)
    surname = fields.CharField(max_length=100)
    email = fields.CharField(max_length=100)
    role = fields.CharEnumField(enum_type=UserRoleEnum, default=UserRoleEnum.user)
    password = fields.CharField(max_length=255)

    def __str__(self):
        return self.username

    class Meta:
        table = 'users'


class Comment(BaseModel):
    user = fields.ForeignKeyField(
        'models.User', related_name='user_comments',
        on_delete=fields.OnDelete.CASCADE
    )
    task = fields.ForeignKeyField(
        'models.Task', related_name='task_comments',
        on_delete=fields.OnDelete.CASCADE
    )
    text = fields.TextField()

    class Meta:
        table = 'comments'


class Category(BaseModel):
    name = fields.CharField(max_length=255)

    def __str__(self):
        return self.name

    class Meta(BaseModel.Meta):
        table = 'categories'
